import pandas as pd
import pyreadstat

import tsapiness.tsapi as ts


class Connection:
    def __init__(self, sav_file):
        self.sav_file = sav_file


class Survey:

    def __init__(self, connection: Connection, id_var: str, date_var: str):
        self.id_variable = id_var
        self.date_variable = date_var
        self.connection = connection
        self.data, self.meta = pyreadstat.read_sav(self.connection.sav_file)
        self.metadata = self.get_metadata(self.meta)
        self.interviews = self.get_interviews(self.data)


    def check_range(self, vi):
        d = self.data[vi]

        v_max = d.values.max()
        v_min = d.values.min()
        value_range_dict = {'from': v_min, 'to': v_max}
        _r = ts.ValueRange(**value_range_dict)
        return _r

    def get_interviews(self, data):
        interviews = []
        raw_interview = pd.DataFrame(data)

        for index, row in raw_interview.iterrows():
            _dict = {'ident': row[self.id_variable],
                     'date': row[self.date_variable],
                     'complete': True,
                     'dataItems': []}
            interview = ts.Interview(**_dict)

            for name, value in row.iteritems():
                if not pd.isna(value):
                    di = ts.DataItem(ident=name, values=[value])
                    interview.data_items.append(di)
            interviews.append(interview)

        return interviews

    def get_metadata(self, data):

        s_name = str(self.connection.sav_file).split('\\')[-1]
        s_title = ''
        _s = ts.SurveyMetadata(name=s_name, title=s_title)
        _s.variables = self.get_variables()

        return _s

    def get_variables(self):
        number_of_variables = self.meta.number_columns
        m = self.meta
        v_list = []

        for index in range(number_of_variables):

            v_ident = m.column_names[index]

            v_type = "single"
            v_use = ""
            v_l = {'text': m.column_names_to_labels[v_ident],
                   'altLabels': [{"mode": "interview",
                                  "text": m.column_names_to_labels[v_ident],
                                  'langIdent': 'EN'}]}
            v_name = ""

            _v = ts.Variable(ident=v_ident, type=v_type, use=v_use,
                             label=v_l, name=v_name)

            if v_ident in m.variable_value_labels:
                # do stuff
                values = []
                for key, value in m.variable_value_labels[v_ident].items():
                    value_item = ts.Value(ident=key,
                                          code=key,
                                          label={'text': value})

                    values.append(value_item)

                _v.variable_values.values = values
            value_range = self.check_range(v_ident)
            _v.variable_values.range = value_range
            v_list.append(_v)
        return v_list






import json
import requests
import tsapiness.tsapi as ts


class Connection:
    def __init__(self, server):
        self.server = server


class Surveys:
    def __init__(self, connection):
        self.connection = connection
        self.surveys = self.get_surveys()

    def __getitem__(self, item):
        return self.surveys[item]

    def __iter__(self):
        return list(self.surveys)

    def __next__(self):
        try:
            result = self.surveys[self.index]
        except IndexError:
            raise StopIteration
        self.index += 1
        return result

    def get_surveys(self):
        r = requests.get(f'{self.connection.server}/Surveys')
        a = json.loads(r.text)
        return a


class Survey:
    def __init__(self, survey_id, connection):
        self.connection = connection
        self.metadata = self.get_survey(survey_id)
        self.interviews = self.get_interviews(survey_id)

    def get_survey(self, s_id):
        url = f'{self.connection.server}/Surveys/{s_id}/Metadata'
        r = requests.get(url)
        json_r = json.loads(r.text)
        survey_obj = ts.SurveyMetadata(**json_r)

        return survey_obj

    def get_interviews(self, s_id):
        headers = {
            'accept': 'application/json',
            # Already added when you pass json=
            # 'Content-Type': 'application/json',
        }

        json_data = {
            'surveyId': s_id,
            'start': 1,
            'maxLength': 100,
            'completeOnly': True,
            'date': '2022-06-01T13:19:58.293Z',
        }

        r = requests.post(
            f'{self.connection.server}/Surveys/{s_id}/Interviews',
            headers=headers, json=json_data)
        json_r = json.loads(r.text)
        interviews = []
        for interview in json_r:
            interview_record = ts.Interview(**interview)
            interviews.append(interview_record)

        return interviews



import xml.etree.ElementTree as et
import tsapiness.tsapi as ts


class Connection:
    def __init__(self, asc_file: str, sss_file: str):
        self.asc_file = asc_file
        self.sss_file = sss_file


class Survey:

    def __init__(self, connection: Connection):
        self.connection = connection
        self.metadata = SurveyMetaData(self.connection.sss_file)
        self.interviews = self.get_interviews(self.connection.asc_file)

    def get_interviews(self, file: str) -> list:
        interviews = []
        with open(file) as f:
            lines = f.readlines()
            count_row_id = 0

            for line in lines:
                # create Interview
                count_row_id += 1
                i = {'ident': count_row_id,
                     'date': None,
                     'complete': True,
                     'dataItems': []
                     }
                iv = ts.Interview(**i)

                # populate dataitems
                for loc in self.metadata.variable_positions:
                    start = 0
                    finish = 0
                    subfields = 0
                    width = 0

                    start = int(loc['start']) - 1
                    finish = int(loc['finish'])
                    if 'subfields' in loc.keys():
                        subfields = int(loc['subfields'])
                    else:
                        subfields = 0
                    if 'width' in loc.keys():
                        width = int(loc['width'])
                    else:
                        width = 0
                    datapoint = line[start:finish]
                    datapoints = []
                    if subfields > 0:
                        datapoints = [datapoint[i:i + width] for i in
                                      range(0, len(datapoint), width)]
                    else:
                        datapoints.append(datapoint)

                    di = ts.DataItem(ident=loc['ident'], values=datapoints)
                    iv.data_items.append(di)
                interviews.append(iv)
        return interviews


def _get_node_attrib(item, attribute, if_none):
    _a = if_none
    if attribute in item.attrib:
        _a = item.attrib[attribute]
        return _a


class SurveyMetaData:
    def __init__(self, file: str):
        self.file = file
        self.tree = et.parse(self.file)
        self.survey = self.get_survey()
        self.variable_positions = self._get_variable_position()

    @property
    def xml_tree(self) -> et.ElementTree:
        """
        converts the xml schema into an xml ElementTree object
        :return: ElementTree
        """
        tree = et.parse(self.file)
        return tree

    def _root(self):
        _root = self.xml_tree.getroot()
        return _root

    def get_value(self, node):
        v_ident = _get_node_attrib(node, 'ident', "")
        v_code = _get_node_attrib(node, 'code', "")
        v_score = _get_node_attrib(node, 'score', 0)
        v_ref = None

        v_label = node.text
        v_label = {'text': v_label}
        _v = ts.Value(label=v_label,
                      ident=v_ident,
                      code=v_code,
                      score=v_score,
                      ref=v_ref)

        return _v

    def get_variable_values(self, node) -> ts.VariableValues:
        # expects the node called values
        # values can have two types of items 1. range, 2. value
        # there will be only one range but an unlimited number of values.
        if node.tag == 'values':
            _variable_value = ts.VariableValues()
            _range = None
            value_list = []
            for val in node.findall('value'):
                _val = self.get_value(val)
                value_list.append(_val)

            for rng in node.findall('range'):
                range_to = _get_node_attrib(rng, 'to', 0)
                range_from = _get_node_attrib(rng, 'from', 0)
                range_dict = {'from': range_from, 'to': range_to}
                _range = ts.ValueRange(**range_dict)
            _variable_value.values = value_list
            _variable_value.range = _range
            return _variable_value
        else:
            pass

    def get_survey(self) -> ts.SurveyMetadata:
        survey_node = self._root().findall('survey')[0]
        s_name = ''
        s_title = ''
        for attr in survey_node:
            if attr.tag == 'name':
                s_name = attr.text
            elif attr.tag == 'title':
                s_title = attr.text
        _s = ts.SurveyMetadata(name=s_name, title=s_title)
        _s.variables = self._get_variable()

        return _s

    def _get_variable_position(self) -> list:
        variable_nodes = self._root().iter('variable')
        v_list = []
        for var in variable_nodes:
            v_ident = _get_node_attrib(var, 'ident', "")
            v_pos = {}
            v_spread = {}
            v_size = 0

            for attr in var:
                v_pos = attr.attrib if attr.tag == 'position' else v_pos
                v_spread = attr.attrib if attr.tag == 'spread' else v_spread
                v_size = attr.attrib if attr.tag == 'size' else v_size
            v_data_location = {'ident': v_ident}
            v_data_location.update(v_pos)
            v_data_location.update(v_spread)
            if v_size > 0:
                v_data_location.update({'size': v_size})
            v_list.append(v_data_location)
        return v_list

    def _get_variable(self) -> list:
        variable_nodes = self._root().iter('variable')
        v_list = []
        for var in variable_nodes:
            v_ident = _get_node_attrib(var, 'ident', "")
            v_type = _get_node_attrib(var, 'type', "")
            v_use = _get_node_attrib(var, 'use', "")
            v_label = ""
            v_name = ""
            v_values = None

            for attr in var:
                v_label = attr.text if attr.tag == 'label' else v_label
                v_name = attr.text if attr.tag == 'name' else v_name
                v_values = self.get_variable_values(
                    attr) if attr.tag == 'values' else v_values

            v_label = {'text': v_label}
            _v = ts.Variable(ident=v_ident, type=v_type, use=v_use,
                             label=v_label, name=v_name)

            _v.variable_values = v_values
            v_list.append(_v)
        return v_list




def add(d, label, obj, apply_to_tsapi=False):
    if apply_to_tsapi:
        if obj is not None:
            d[label] = obj.to_tsapi()
        return d
    else:
        if obj is not None:
            d[label] = obj
        return d


def parse(list_to_parse: list, obj: object) -> list:
    list_to_return = []
    if list_to_parse is not None:
        for list_item in list_to_parse:
            list_item_obj = obj(**list_item)
            list_to_return.append(list_item_obj)
    return list_to_return


def flatten_variable(variable, variable_list):
    if len(variable.looped_variables) > 0 and len(variable.values) > 0:
        for value in variable.values:
            _a = variable.to_dict()
            _a.update(value.to_dict())
            variable_list.append(_a)
        for loop_variable in variable.looped_variable_values:
            flatten_variable(variable=loop_variable,
                             variable_list=variable_list)
    elif len(variable.looped_variables) == 0 and len(variable.values) > 0:
        _a = variable.to_dict()
        for value in variable.values:
            _a = variable.to_dict()
            _a.update(value.to_dict())
            variable_list.append(_a)
    elif len(variable.looped_variables) > 0 and len(variable.values) == 0:
        # check if this is valid
        pass

    elif len(variable.looped_variables) == 0 and len(variable.values) == 0:
        _a = variable.to_dict()
        variable_list.append(_a)
    else:
        _a = variable.to_dict()
        variable_list.append(_a)
    if len(variable.otherSpecifyVariables) > 0:
        for osv in variable.otherSpecifyVariables:
            flatten_variable(variable=osv, variable_list=variable_list)

    return variable_list


class SurveyMetadata:
    def __init__(self, hierarchies=None, name="", title="", interviewCount=0,
                 languages=None, notAsked="", noAnswer="", variables=None,
                 sections=None):
        self.hierarchies = parse(hierarchies, Hierarchy)
        self.name = name
        self.title = title
        self.interview_count = interviewCount
        self.not_asked = notAsked
        self.no_answer = noAnswer
        self.variables = parse(variables, Variable)
        self.sections = parse(sections, Section)
        self.languages = parse(languages, Language)

    def __str__(self):
        return f'Name: {self.name}, Title {self.title}'

    def __repr__(self):
        return f'Survey({self.name})'

    def get_variables_list(self):
        variable_objects = ['variables',
                            'looped_variable',
                            'other_specify_variable',
                            ]
        objects_with_variables = ['survey', 'sections', 'hierarchies', ]
        objects_with_variables += variable_objects

        _dict = {}
        _dict = vars(self)

        return _dict

    def to_tsapi(self):
        _dict = {
            'hierarchies': [h.to_tsapi() for h in self.hierarchies],
            'name': self.name,
            'title': self.title,
            'interviewCount': self.interview_count,
            'languages': [lang.to_tsapi() for lang in self.languages],
            'notAsked': self.not_asked,
            'noAnswer': self.no_answer,
            'variables': [v.to_tsapi() for v in self.variables],
            'sections': [sect.to_tsapi() for sect in self.sections]
        }
        return _dict


class Section:
    def __init__(self, label, variables):
        self.label = Label(**label)
        self.sections = ""
        self.variables = parse(variables, Variable)

    def __str__(self):
        return f'{self.label}'

    def __repr__(self):
        return f'{self.label}'

    def to_tsapi(self):
        _dict = {
            'label': self.label.to_tsapi(),
            'variables': [v.to_tsapi() for v in self.variables]
        }
        return _dict


class Label:
    def __init__(self, text, altLabels=None):

        self.text = text
        self.alt_labels = parse(altLabels, AltLabel)

    def __str__(self):
        return self.text

    @property
    def label_analysis(self) -> str:
        alt_text = ""
        if self.alt_labels:
            for alts in self.alt_labels:
                if alts.mode == 'analysis':
                    alt_text = alts.text
                    return alt_text
        return alt_text

    @property
    def label_interview(self) -> str:
        alt_text = ""
        if self.alt_labels:
            for alts in self.alt_labels:
                if alts.mode == 'interview':
                    return alts.text
        return alt_text

    def to_tsapi(self):
        _dict = {}
        _dict = add(_dict, 'text', self.text)
        if len(self.alt_labels) > 0:
            _dict['altLabels'] = [al.to_tsapi() for al in self.alt_labels]
        return _dict


class Variable:
    def __init__(self,
                 ordinal=0,
                 label=None,
                 name="",
                 ident="",
                 type="",
                 values=None,
                 use="",
                 maxResponses=0,
                 loopedVariables=None,
                 otherSpecifyVariables=None):

        self.ident: str = ident
        self.ordinal: int = ordinal
        self.type: str = type
        self.name: str = name
        self.label: Label = Label(**label)
        self.use: str = use
        self.maxResponses: int = maxResponses
        self.otherSpecifyVariables = parse(otherSpecifyVariables,
                                           OtherSpecifyVariable)
        if values is None:
            values = {}
        self.variable_values = VariableValues(**values)

        self.looped_variables = parse(loopedVariables, LoopedVariable)

    def to_tsapi(self):

        _dict = {}
        _dict = add(_dict, 'ordinal', self.ordinal)
        _dict = add(_dict, 'label', self.label, True)
        _dict = add(_dict, 'name', self.name)
        _dict = add(_dict, 'ident', self.ident)
        _dict = add(_dict, 'type', self.type)
        _dict = add(_dict, 'values', self.variable_values, True)
        _dict = add(_dict, 'use', self.use)
        _dict = add(_dict, 'maxResponses', self.maxResponses)
        if len(self.looped_variables) > 0:
            _dict['loopedVariables'] = [lv.to_tsapi()
                                        for lv in self.looped_variables],
        if len(self.otherSpecifyVariables) > 0:
            _dict['otherSpecifyVariables'] = [o.to_tsapi() for o in
                                              self.otherSpecifyVariables],

        return _dict

    @property
    def looped_variable_values(self):
        looped_variable_value_list = []
        if len(self.values) == 0 | len(self.looped_variables) == 0:
            return looped_variable_value_list

        for value in self.values:
            for l_v in self.looped_variables:
                looped_variable_value_list.append(l_v)
        return looped_variable_value_list

    def __str__(self):
        return f'{self.ident}'

    def __repr__(self):
        return f'{self.ident}'

    @property
    def alt_labels(self):
        return self.label.alt_labels

    @property
    def label_text(self):
        return self.label.text

    @property
    def values(self):
        return self.variable_values.values

    @property
    def label_interview(self):
        return self.label.label_interview

    @property
    def label_analysis(self):
        return self.label.label_analysis

    @property
    def range_from(self):
        _r = ""
        if self.variable_values.range:
            _r = self.variable_values.range.range_from
        return _r

    @property
    def range_to(self):
        _r = ""
        if self.variable_values.range:
            _r = self.variable_values.range.range_to
        return _r

    def range(self):
        return self.variable_values.range

    def to_dict(self):
        _dict = {
            'var_name': self.name,
            'variable_name': self.name,
            'variable_ident': self.ident,
            'variable_label': self.label_text,
            'variable_type': self.type,
            'variable_interview_label': self.label_interview,
            'variable_analysis_label': self.label_analysis,
        }
        if self.range:
            _dict['variable_range_from'] = self.range_from
            _dict['variable_range_to'] = self.range_to
        return _dict


class Language:
    def __init__(self, ident="", name="", subLanguages=None):
        # if subLanguages is None:
        #     subLanguages = []
        self.ident = ident
        self.name = name
        self.sub_languages = parse(subLanguages, Language)

    def __str__(self):
        return f'{self.name}'

    def __repr__(self):
        return f'language:{self.name}'

    def to_tsapi(self):
        _dict = {
            'ident': self.ident,
            'name': self.name
        }
        if len(self.sub_languages) > 0:
            _dict['subLanguage'] = [lang.to_tsapi()
                                    for lang in self.sub_languages]
        return _dict


class AltLabel:
    def __init__(self, mode='interview', text="", langIdent=""):
        self.mode = mode
        self.text = text
        self.langIdent = langIdent

    def __str__(self):
        return self.text

    def __repr__(self):
        return f'{self.mode} {self.text} {self.langIdent}'

    def to_tsapi(self):
        _dict = {
            'mode': self.mode,
            'text': self.text,
            'langIdent': self.langIdent
        }
        return _dict


class ValueRange:
    def __init__(self, **kwargs):
        self.range_from = kwargs['from']
        self.range_to = kwargs['to']

    def __repr__(self):
        return f'range: {self.range_from} - {self.range_to}'

    def to_tsapi(self):
        _dict = {
            'from': self.range_from,
            'to': self.range_to
        }
        return _dict


class ValueRef:
    def __init__(self, variableIdent="", valueIdent=""):
        self.variable_ident = variableIdent
        self.value_ident = valueIdent

    def __str__(self):
        a = f'variable_ident:{self.variable_ident}, ' \
            f'value_ident:{self.value_ident} '
        return a

    def __repr__(self):
        a = f'variable_ident:{self.variable_ident}, ' \
            f'value_ident:{self.value_ident} '
        return a

    def to_tsapi(self):
        _dict = {
            'variable_ident': self.variable_ident,
            'value_ident': self.value_ident
        }
        return _dict


class Value:
    def __init__(self, ident="", code="", label=None, score=0, ref=None):
        if ref is None:
            ref = {}
        if label is None:
            label = []

        self.ident = ident
        self.code = code
        self.label = Label(**label)
        self.score = score

        self.ref = None  # ValueRef(**ref)

    def to_tsapi(self):
        _dict = {}
        _dict = add(_dict, 'ident', self.ident)
        _dict = add(_dict, 'code', self.code)
        _dict = add(_dict, 'label', self.label, True)
        _dict = add(_dict, 'score', self.score)
        _dict = add(_dict, 'ref', self.ref)

        return _dict

    # @property
    # def label(self):
    #     return f'{self.ident} - {self._label.text}'

    def __str__(self):
        return f'{self.ident} - {self.label.text}'

    def __repr__(self):
        return self.label

    def to_dict(self):
        _dict = {
            'value_code': self.code,
            'value_ident': self.ident,
            'value_label': self.label.text,
            'value_score': self.score}
        return _dict


class VariableValues:
    def __init__(self, range=None, values=None):
        self.range = range

        self.values = parse(values, Value)

        if self.range is not None:
            self.range = ValueRange(**self.range)

    def to_tsapi(self):
        _dict = {}
        if self.values is not None:
            _dict['values'] = [val.to_tsapi() for val in self.values]
        if self.range is not None:
            _dict['range'] = self.range.to_tsapi()
        return _dict


class OtherSpecifyVariable(Variable):
    def __init__(self,
                 ordinal=0,
                 label=None,
                 name="",
                 ident="",
                 type="",
                 values=None,
                 use="",
                 maxResponses=0,
                 loopedVariables=None,
                 otherSpecifyVariables=None,
                 parentValueIdent=""):
        super().__init__(ordinal=ordinal,
                         label=label,
                         name=name,
                         ident=ident,
                         type=type,
                         values=values,
                         use=use,
                         maxResponses=maxResponses,
                         loopedVariables=loopedVariables,
                         otherSpecifyVariables=otherSpecifyVariables)
        self.parentValueIdent = parentValueIdent

    def to_tsapi(self):
        _dict = {}
        _dict = add(_dict, 'ordinal', self.ordinal)
        _dict = add(_dict, 'label', self.label, True)
        _dict = add(_dict, 'name', self.name)
        _dict = add(_dict, 'ident', self.ident)
        _dict = add(_dict, 'type', self.type)
        _dict = add(_dict, 'values', self.variable_values, True)
        _dict = add(_dict, 'use', self.use)
        _dict = add(_dict, 'maxResponses', self.maxResponses)
        _dict['loopedVariables'] = [lv.to_tsapi() for lv in
                                    self.looped_variables]
        _dict['otherSpecifyVariables'] = [o.to_tsapi() for o in
                                          self.otherSpecifyVariables]

        _dict = add(_dict, 'parentValueIdent', self.parentValueIdent)

        return _dict


class LoopedVariable(Variable):

    def __init__(self,
                 ordinal=0,
                 label=None,
                 name="",
                 ident="",
                 type="",
                 values=None,
                 use="",
                 maxResponses=0,
                 loopedVariables=None,
                 otherSpecifyVariables=None,
                 loop_ref=None):
        super().__init__(ordinal=ordinal,
                         label=label,
                         name=name,
                         ident=ident,
                         type=type,
                         values=values,
                         use=use,
                         maxResponses=maxResponses,
                         loopedVariables=loopedVariables,
                         otherSpecifyVariables=otherSpecifyVariables)

        self.loop_ref = loop_ref

    def to_tsapi(self):
        _dict = {}
        _dict = add(_dict, 'ordinal', self.ordinal)
        _dict = add(_dict, 'label', self.label, True)
        _dict = add(_dict, 'name', self.name)
        _dict = add(_dict, 'ident', self.ident)
        _dict = add(_dict, 'type', self.type)
        _dict = add(_dict, 'values', self.variable_values, True)
        _dict = add(_dict, 'use', self.use)
        _dict = add(_dict, 'maxResponses', self.maxResponses)
        _dict['loopedVariables'] = [lv.to_tsapi() for lv in
                                    self.looped_variables]
        _dict['otherSpecifyVariables'] = [o.to_tsapi() for o in
                                          self.otherSpecifyVariables]
        _dict = add(_dict, 'loopRef', self.loop_ref, True)

        return _dict

    @property
    def parent_variable_ident(self):
        if self.loop_ref is not None:
            return self.loop_ref.variable_ident
        else:
            return ""

    @property
    def parent_value_ident(self):
        if self.loop_ref is not None:
            return self.loop_ref.value_ident
        else:
            return ""

    def to_dict(self):
        _dict = {
            'var_name': self.name,
            'variable_name': self.name,
            'variable_ident': self.ident,
            'variable_label': self.label_text,
            'variable_type': self.type,
            'variable_interview_label': self.label_interview,
            'variable_analysis_label': self.label_analysis,
            'parent_variable_label': self.parent_variable_ident,
            'parent_value_label': self.parent_value_ident,

        }
        if self.range:
            _dict['variable_range_from'] = self.range_from
            _dict['variable_range_to'] = self.range_to
        return _dict

    def __str__(self):
        return f'{self.name} - {self.parent_value_ident}'

    def __repr__(self):
        return f'{self.name}'


class Hierarchy:
    def __init__(self, ident: str = "", parent=None, metadata=None):
        if metadata is None:
            metadata = {}
        if parent is None:
            parent = {}
        self.ident: str = ident
        self.parent: ParentDetails = parent
        self.metadata: MetaData = metadata

    def to_tsapi(self):
        _dict = {
            'ident': self.ident,
            'metadata': self.metadata.to_tsapi(),
            'parent': self.parent
        }


class ParentDetails:
    def __init__(self, level, linkVar, ordered):
        self.level = level
        self.link_var = linkVar
        self.ordered = ordered


class MetaData:
    def __init__(self, name="", title="", interviewCount=0, languages=None,
                 notAsked="", noAnswer="", variables=None, sections=None):
        self.name = name
        self.title = title
        self.interview_count = interviewCount
        self.not_asked = notAsked
        self.no_answer = noAnswer
        self.sections = parse(sections, Section)
        self.variables = parse(variables, Variable)
        self.languages = parse(languages, Language)

    def to_tsapi(self):
        _dict = {
            'name': self.name,
            'title': self.name,
            'interviewCount': self.interview_count,
            'notAsked': self.not_asked,
            'noAnswer': self.no_answer,
            'variables': [var.to_tsapi() for var in self.variables],
            'sections': [sect.to_tsapi() for sect in self.sections],
            'languages': [lang.to_tsapi() for lang in self.languages],
        }
        return _dict


class InterviewsQuery:
    def __init__(self,
                 surveyId="",
                 start=0,
                 maxLength=0,
                 completeOnly=True,
                 variables=None,
                 interviewIdents=None,
                 date=""):
        self.survey_id = surveyId
        self.start = start
        self.max_length = maxLength
        self.complete_only = completeOnly
        self.variables = variables
        self.interview_idents = interviewIdents
        self.date = date


class LoopedDataItem:
    def __init__(self,
                 parent="",
                 ident="",
                 values=None,
                 loopedDataItems=None,
                 ):
        self.parent = parent
        self.ident = ident
        self.values = [v for v in values]

        if loopedDataItems is not None:
            self.looped_data_items = [LoopedDataItem(**lr)
                                      for lr in loopedDataItems]
        else:
            self.looped_data_items = None

    def to_tsapi(self):
        _dict = {'parent': self.parent,
                 'ident': self.ident,
                 'values': self.values}
        if self.looped_data_items:
            _dict['loopedDataItems'] = [ldi.to_tsapi() for ldi in
                                        self.looped_data_items]
        return _dict


class DataItem:
    def __init__(self, ident="", values=None, loopedDataItems=None):
        self.ident = ident
        self.values = [v for v in values]
        self.looped_data_items = []
        if loopedDataItems is not None:
            self.looped_data_items = [LoopedDataItem(**lr)
                                      for lr in loopedDataItems]
        else:
            self.looped_data_items = None

    def to_tsapi(self):
        _dict = {'ident': self.ident,
                 'values': self.values}

        if self.looped_data_items:
            _dict['loopedDataItems'] = [ldi.to_tsapi()
                                        for ldi in self.looped_data_items]

        return _dict


class Level:
    def __init__(self, ident=""):
        self.ident = ident

    def to_tsapi(self):
        _dict = {'ident': self.ident}
        return _dict


class HierarchicalInterview:
    def __init__(self, level=None, ident="", date="", complete=True,
                 dataItems=None, hierarchicalInterviews=None):
        self.level = Level(**level)
        self.ident = ident
        self.date = date
        self.complete = complete
        self.data_items = parse(dataItems, DataItem)
        self.hierarchical_interviews = parse(hierarchicalInterviews,
                                             HierarchicalInterview)

    def to_tsapi(self):
        _dict = {'level': self.level.to_tsapi(),
                 'ident': self.ident,
                 'date': self.date,
                 'complete': self.complete,
                 'dataItems': [di.to_tsapi() for di in self.data_items],
                 'hierarchicalInterviews': [hi.to_tsapi() for hi in
                                            self.hierarchical_interviews]}
        return _dict


class Interview:
    def __init__(self, ident="", date="", complete=True, dataItems=None,
                 hierarchicalInterviews=None):
        self.ident = ident
        self.date = date
        self.complete = complete
        self.data_items = parse(dataItems, DataItem)
        self.hierarchical_interviews = parse(hierarchicalInterviews,
                                             HierarchicalInterview)

    def to_tsapi(self):
        _dict = {'ident': self.ident,
                 'date': self.date,
                 'complete': self.complete,
                 'dataItems': [di.to_tsapi() for di in self.data_items]}
        if self.hierarchical_interviews:
            _dict['hierarchicalInterviews'] = [hi.to_tsapi() for hi in
                                               self.hierarchical_interviews]

        return _dict
